'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = aframeVideoBillboardComponent;

var _cuid = require('cuid');

var _cuid2 = _interopRequireDefault(_cuid);

var _videoStream = require('./video-stream');

var _videoStream2 = _interopRequireDefault(_videoStream);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PERMISSION_DENIED_EVENT = 'video-permission-denied';
var PLAY_EVENT = 'video-play';

function createVideoElementAsAsset(id) {

  var video = document.querySelector('#' + id);

  if (!video) {
    video = document.createElement('video');
  }

  video.setAttribute('id', id);
  video.setAttribute('autoplay', true);
  video.setAttribute('src', '');

  var assets = document.querySelector('a-assets');

  if (!assets) {
    assets = document.createElement('a-assets');
    document.querySelector('a-scene').appendChild(assets);
  }

  if (!assets.contains(video)) {
    assets.appendChild(video);
  }

  return video;
}

function shrinkwrapMinDimensions(_ref, _ref2) {
  var minWidth = _ref.width;
  var minHeight = _ref.height;
  var width = _ref2.width;
  var height = _ref2.height;


  var aspectRatio = width / height;

  // assume width is exact, and height is taller than minHeight
  var shrunkWidth = minWidth;
  var shrunkHeight = shrunkWidth / aspectRatio;

  if (shrunkHeight < minHeight) {
    // our assumption was wrong, so we need to grow the shrunk sizes to make
    // height exact
    shrunkHeight = minHeight;
    shrunkWidth = shrunkHeight * aspectRatio;
  }

  return {
    width: shrunkWidth,
    height: shrunkHeight
  };
}

/**
 * @param aframe {Object} The Aframe instance to register with
 * @param componentName {String} The component name to use. Default: 'video-billboard'
 */
function aframeVideoBillboardComponent(aframe, componentName) {

  /**
   * Draggable component for A-Frame.
   */
  aframe.registerComponent(componentName, {
    schema: {
      /*
       * @param {string} [deviceId=null] - Select the specific device for
       * display. Note that if it is not a valid video device, nothing will be
       * shown.
       */
      deviceId: { default: null },

      /*
       * @param {number} [minWidth=4] - The minimum width in world-units to
       * display the video at. Video aspect ratio will be preserved.
       */
      minWidth: { default: 4 },

      /*
       * @param {number} [minHeight=3] - The minimum height in world-units to
       * display the video at. Video aspect ratio will be preserved.
       */
      minHeight: { default: 3 }
    },

    /**
     * Called once when component is attached. Generally for initial setup.
     */
    init: function init() {
      this._videoId = (0, _cuid2.default)();
      this._activeVideo = {
        source: null,
        stream: null
      };
    },


    /**
     * Called when component is attached and when component data changes.
     * Generally modifies the entity based on the data.
     */
    update: function update() {
      var _this = this;

      (0, _videoStream2.default)(this.data.deviceId).then(function (_ref3) {
        var source = _ref3.source;
        var stream = _ref3.stream;


        _this._activeVideo.soure = source;
        _this._activeVideo.stream = stream;

        // Creating an aframe asset out of a new video tag
        var videoEl = createVideoElementAsAsset(_this._videoId);
        var entityEl = _this.el;

        var onLoadedMetaData = function onLoadedMetaData(_) {

          // Only want this event listener to execute once
          videoEl.removeEventListener('loadeddata', onLoadedMetaData);

          videoEl.play();

          // Pointing this aframe entity to that video as its source
          entityEl.setAttribute('src', '#' + _this._videoId);

          var _shrinkwrapMinDimensi = shrinkwrapMinDimensions({ width: _this.data.minWidth, height: _this.data.minHeight }, { width: videoEl.videoWidth, height: videoEl.videoHeight });

          var width = _shrinkwrapMinDimensi.width;
          var height = _shrinkwrapMinDimensi.height;

          // Set the width and height correctly

          entityEl.setAttribute('width', width);
          entityEl.setAttribute('height', height);

          entityEl.emit(PLAY_EVENT, { source: source, stream: stream });
        };

        videoEl.addEventListener('loadeddata', onLoadedMetaData);

        // And starting the video streaming
        videoEl.srcObject = stream;

        _this._videoElement = videoEl;
      }).catch(function (error) {
        // TODO: Check error.name for 'PermissionDeniedError'?
        // https://developer.mozilla.org/en-US/docs/Web/API/MediaDevices/getUserMedia
        _this.el.emit(PERMISSION_DENIED_EVENT, { error: error });
      });
    },


    /**
     * Called when a component is removed (e.g., via removeAttribute).
     * Generally undoes all modifications to the entity.
     */
    remove: function remove() {
      if (this._videoElement) {
        this._videoElement.parentNode.removeChild(this._videoElement);
        this._videoElement = null;
      }
    },


    /**
     * Called when entity pauses.
     * Use to stop or remove any dynamic or background behavior such as events.
     */
    pause: function pause() {
      if (this._videoElement) {
        this._videoElement.pause();
      }
    },


    /**
     * Called when entity resumes.
     * Use to continue or add any dynamic or background behavior such as events.
     */
    play: function play() {
      if (this._videoElement) {
        this._videoElement.play();
      }
    },
    getDevices: function getDevices() {
      return (0, _videoStream.askPermission)().then(_videoStream.getDevices);
    },
    getActiveDevice: function getActiveDevice() {
      return this.activeVideo.source;
    }
  });
}